package org.bitbucket.bradleysmithllc.etlagent.informatica.handler;

import com.fasterxml.jackson.databind.JsonNode;
import com.google.gson.GsonBuilder;
import org.bitbucket.bradleysmithllc.etlagent.dto.GenericResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.InformaticaRequestHandler;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.*;
import org.bitbucket.bradleysmithllc.etlunit.BasicRuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.LoggingProcessExecutor;
import org.bitbucket.bradleysmithllc.etlunit.PrintWriterLog;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.feature.FeatureOperation;

public abstract class AbstractHandler<T, J extends GenericResponseDTO> implements InformaticaRequestHandler<T, J>
{
	protected RuntimeSupport runtimeSupport;

	@Override
	public GsonBuilder configureBuilder(GsonBuilder builder) {
		return builder;
	}

	protected PrintWriterLog log = new PrintWriterLog();

	private FeatureOperation featureOperation;
	protected InformaticaConfiguration informaticaConfiguration;

	@Override
	public void setInformaticaConfiguration(InformaticaConfiguration informaticaConfiguration) {
		this.informaticaConfiguration = informaticaConfiguration;
	}

	@Override
	public void receiveRuntimeSupport(RuntimeSupport runtimeSupport) {
		this.runtimeSupport = runtimeSupport;
	}

	@Override
	public final void setFeatureOperation(FeatureOperation featureOperation) {
		this.featureOperation = featureOperation;
	}

	@Override
	public final JsonNode getValidator() {
		return featureOperation.getSignatures().get(0).getValidatorNode();
	}

	public interface RepositoryVisitor
	{
		public void useClient(InformaticaRepositoryClient client, RuntimeSupport runtimeSupport) throws Exception;
	}

	protected void useRepository(InformaticaRepository repository, RepositoryVisitor visitor) throws Exception {
		InformaticaRepositoryClient informaticaRepositoryClient = repository.getInformaticaRepositoryClient();

		try
		{
			visitor.useClient(informaticaRepositoryClient, runtimeSupport);
		} finally
		{
			informaticaRepositoryClient.dispose();
		}
	}

	public interface IntegrationServiceVisitor
	{
		public void useClient(InformaticaIntegrationServiceClient client, RuntimeSupport runtimeSupport) throws Exception;
	}

	protected void useIntegrationService(InformaticaRepository repository, InformaticaIntegrationService intsvc, IntegrationServiceVisitor visitor) throws Exception {
		InformaticaIntegrationServiceClient informaticaRepositoryClient = intsvc.getInformaticaIntegrationServiceClient();

		try
		{
			visitor.useClient(informaticaRepositoryClient, runtimeSupport);
		} finally
		{
			informaticaRepositoryClient.dispose();
		}
	}
}