package org.bitbucket.bradleysmithllc.etlagent.informatica.handler;

import com.fasterxml.jackson.databind.JsonNode;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.PurgeTestFoldersResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.json.informatica.purge_test_folders.PurgeTestFoldersRequest;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaDomain;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepository;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepositoryClient;
import org.bitbucket.bradleysmithllc.etlagent.resources.ContextBase;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

public class PurgeTestFolders extends AbstractHandler<PurgeTestFoldersRequest, PurgeTestFoldersResponseDTO>
{
	@Override
	public String getId() {
		return "purgeTestFolders";
	}

	@Override
	public PurgeTestFoldersRequest getRequestContainerObject() {
		return new PurgeTestFoldersRequest();
	}

	@Override
	public PurgeTestFoldersResponseDTO process(JsonNode request, PurgeTestFoldersRequest container) throws Exception {
		System.out.println("Purging test folders");

		String domain = container.getInformaticaDomain();

		String repo = container.getInformaticaRepository();

		// resolve to an informatica domain
		InformaticaDomain informaticaDomain = informaticaConfiguration.getDefaultDomain();

		if (domain != null)
		{
			System.out.println("No domain specified - using the default domain");

			informaticaDomain = informaticaConfiguration.getDomain(domain);

			if (informaticaDomain == null)
			{
				throw new IllegalArgumentException("Domain '" + domain + "' not found");
			}
		}
		else
		{
			System.out.println("No domain specified - using the default domain " + informaticaDomain.getDomainName());
		}

		InformaticaRepository informaticaRepository = null;

		if (repo == null)
		{
			informaticaRepository = informaticaDomain.getDefaultRepository();
		}
		else
		{
			informaticaRepository = informaticaDomain.getRepository(repo);
		}

		if (informaticaRepository == null)
		{
			throw new IllegalArgumentException("Domain '" + domain + "' does not contain repository '" + repo + "'");
		}

		final PurgeTestFoldersResponseDTO dto = new PurgeTestFoldersResponseDTO();

		useRepository(informaticaRepository, new RepositoryVisitor() {
			@Override
			public void useClient(InformaticaRepositoryClient client, RuntimeSupport runtimeSupport) throws Exception {
				int folders = client.purgeTestFolders();

				dto.setNumFoldersRemoved(folders);
				dto.setResponseMessage("Removed [" + folders + "] folders");
			}
		});

		return dto;
	}
}